#include "Logging.h"
#include "AgentManager.h"
#include <resip/stack/SipMessage.hxx>
#include <resip/stack/Pidf.hxx>
#include <resip/dum/ClientSubscription.hxx>
#include <resip/stack/XMLCursor.hxx>
using namespace resip;
using namespace Dumais::JSON;

AgentManager::AgentManager(UserAgent *userAgent, int expire){
    mSubscribeExpire = expire;
    this->mUserAgent = userAgent;
    this->mUserAgent->getDUM()->addClientSubscriptionHandler("presence", this);
}


AgentManager::~AgentManager(){
    //TODO: remove subscription handler
}


void AgentManager::work()
{
    for (AgentList::iterator it=mAgentList.begin();it!=mAgentList.end();it++)
    {
        Agent *agent = it->second;
        if (agent->mAgentState==SubscribeExpired)
        {   
            this->subscribeAgent(agent);
        }
    }
}

Agent* AgentManager::getAgentWithDevice(resip::NameAddr addr)
{
//    std::string user1 = addr.uri().user().data();

    for (AgentList::iterator it=mAgentList.begin();it!=mAgentList.end();it++)
    {
        Agent *agent = it->second;
        if (agent->isForDevice(addr)) return agent;
    }

    return 0;
}


bool AgentManager::subscribeAgent(Agent *agent)
{
    if (!mUserAgent->isRegistered()) return false;
    if (agent->mAgentState==LoggedOut) return false;

    Logging::log("Subscribing agent %s\r\n",agent->getUserName().c_str());
    NameAddr device = agent->mDevice;
    SharedPtr<SipMessage> subMessage = this->mUserAgent->getDUM()->makeSubscription(device, "presence");
    subMessage->header(h_Accepts).push_back(resip::Mime("application","dialog-info+xml"));
    subMessage->header(h_Expires).value()=mSubscribeExpire;
    this->mUserAgent->getDUM()->send( subMessage );

    agent->mAgentState = Subscribing;

    return true;
}

void AgentManager::onUpdatePending(resip::ClientSubscriptionHandle, const resip::SipMessage &notify, bool outOfOrder)
{
    Logging::log("AgentManager::onUpdatePending\r\n");
}
    

void AgentManager::onUpdateActive(resip::ClientSubscriptionHandle h, const resip::SipMessage &msg, bool outOfOrder)
{
    Logging::log("Agent::onUpdateActive\r\n");
    h->acceptUpdate();
    std::string presenceStatus="terminated";

    if(msg.getContents())
    {
        Data data = msg.getContents()->getBodyData();
        try
        {
            XMLCursor cursor(ParseBuffer(data.data(),data.size()));
            if (cursor.firstChild())
            {
                while (cursor.getTag()!="dialog")
                {
                    cursor.nextSibling();
                }            
                if (cursor.firstChild())
                {
                    while (cursor.getTag()!="state")
                    {
                        cursor.nextSibling();
                    }
                    if (cursor.firstChild())
                    {
                        presenceStatus = cursor.getValue().c_str();  //configrmed, early or terminated
                        
                    }
                }             
            }        

        } catch (ParseException &e) {
        }
    }



    Agent *a=0;
    NameAddr to = msg.header(h_From);
    for (AgentList::iterator it=mAgentList.begin();it!=mAgentList.end();it++)
    {

        if (it->second->mDevice == to)
        {
            a = it->second;
            break;
        }
    }

    if (a)
    {
        if (presenceStatus=="confirmed" || presenceStatus=="early")
        {
            a->onPresence(false);
        }
        else if (presenceStatus=="terminated")
        {
            a->onPresence(true);
        }
    }

}
void AgentManager::onUpdateExtension(resip::ClientSubscriptionHandle, const resip::SipMessage &notify, bool outOfOrder)
{
    Logging::log("AgentManager::onUpdateExtension\r\n");
}

int AgentManager::onRequestRetry(resip::ClientSubscriptionHandle, int retrySeconds, const resip::SipMessage &notify)
{
    Logging::log("AgentManager::onRequestRetry\r\n");
}

void AgentManager::onTerminated(resip::ClientSubscriptionHandle, const resip::SipMessage *msg)
{
    NameAddr from = msg->header(h_From);
    std::string user = from.uri().user().data();


    for (AgentList::iterator it=mAgentList.begin();it!=mAgentList.end();it++)
    {
        Agent *agent = it->second;
        if (agent->getAssociatedDevice()==user)
        {
            Logging::log("AgentManager::onTerminated for %s\r\n",user.c_str());
            agent->setAgentState(LoggedOut);
        }
    }
}

void AgentManager::onNewSubscription(resip::ClientSubscriptionHandle cs, const resip::SipMessage &msg)
{
    NameAddr from = msg.header(h_From);
    std::string user = from.uri().user().data();


    for (AgentList::iterator it=mAgentList.begin();it!=mAgentList.end();it++)
    {
        Agent *agent = it->second;
        if (agent->getAssociatedDevice() == user)
        {
            Logging::log("AgentManager::NewSubscription for %s\r\n",user.c_str());
            agent->mSubscriptionHandle = cs->getHandle();
        }
    }

}

void AgentManager::onReadyToSend(resip::ClientSubscriptionHandle, resip::SipMessage &msg)
{
}
void AgentManager::onNotifyNotReceived(resip::ClientSubscriptionHandle)
{
    Logging::log("AgentManager::onNotifyNotReceived\r\n");
}
void AgentManager::onFlowTerminated(resip::ClientSubscriptionHandle)
{
    Logging::log(" AgentManager::onFlowTerminated\r\n");
}


void AgentManager::addAgent(Agent* agent)
{
    //TODO: dont add if key already exists
    mAgentList[agent->mUsername] = agent;
}

Agent* AgentManager::getAgent(std::string agent)
{
    if (mAgentList.find(agent)!=mAgentList.end())
    {
        return mAgentList[agent];
    }
    return 0;
}

Agent* AgentManager::getMostIdleAgentForQueue(Queue *q)
{
    Agent *a=0;
    time_t idle=0;
    for (AgentList::iterator it=mAgentList.begin();it!=mAgentList.end();it++)
    {
        Agent *agent = it->second;
        if (agent->memberOf(q) && agent->getAgentState()==Idle)
        {
            //check for most idle time
            if (agent->getIdleTime() > idle)
            {
                a = agent;
                idle = agent->getIdleTime();
            }
            
        }
    }
    return a;
}

void AgentManager::toJSON(Dumais::JSON::JSON& json)
{
    JSON j = json.addList("agents");
    for (AgentList::iterator it=mAgentList.begin();it!=mAgentList.end();it++)
    {
        Agent *a = it->second;
        a->toJSON(j);
    }
}
